import _ from "lodash"

/**
 * 创建一个排序函数，用于根据指定的键和排序顺序对数组进行排序。
 * @param {boolean | "desc" | "asc"} isAscending 排序顺序，可以是布尔值或字符串 'asc' 或 'desc'。true 或 'asc' 表示升序，false 或 'desc' 表示降序。
 * @param {string | number} key 用于排序的键，可以是字符串或数字。
 * @returns {Function} 一个用于排序的比较函数。
 */
export const createSortFunction = (
  isAscending: boolean | "desc" | "asc",
  key: string | number
) => {
  if (
    (typeof isAscending === "string" && isAscending === "asc") ||
    (typeof isAscending === "boolean" && isAscending)
  ) {
    return (d1: any, d2: any) => (d1?.[key] ?? d1) - (d2?.[key] ?? d2)
  }
  return (d1: any, d2: any) => (d2?.[key] ?? d2) - (d1?.[key] ?? d1)
}

/**
 * 解析 JSON 字符串并返回解析后的对象。
 * @param {any} jsonString 要解析的 JSON 字符串。
 * @param {any} [defaultValue] 解析失败时返回的默认值。默认为输入字符串。
 * @returns {any} 解析后的 JSON 对象，如果解析失败则返回默认值。
 */
export const parseJsonString = (jsonString: any, defaultValue?: any) => {
  const failValue = defaultValue || jsonString
  if (typeof jsonString !== "string") return failValue
  try {
    return JSON.parse(jsonString)
  } catch {
    return failValue
  }
}

/**
 * 递归解析 JSON 对象并返回一个包含所有解析值的新对象。
 * @param {any} jsonObject 要解析的 JSON 对象。
 * @param {any} [defaultValue] 解析失败时返回的默认值。默认为输入对象。
 * @returns {any} 一个包含所有解析值的新对象。
 */
export const deepParseJson = (jsonObject: any, defaultValue?: any) => {
  const finalObject: any = _.isArray(jsonObject) ? [] : {}
  const parse = (obj: any, keyList: (string | number)[]) => {
    if (typeof obj !== "object" || obj === null) return
    const handler = (val: any, key: string | number) => {
      const currentKeyList = [...keyList, key]
      const currentValue = parseJsonString(val)
      _.set(finalObject, currentKeyList, currentValue)
      parse(currentValue, currentKeyList)
    }
    if (_.isArray(obj)) {
      obj.forEach(handler)
    } else {
      _.forOwn(obj, handler)
    }
  }
  parse(parseJsonString(jsonObject, defaultValue), [])
  return finalObject
}

/**
 * 安全地将对象转换为 JSON 字符串。
 * 如果转换失败，则返回对象的字符串表示。
 * @param {any} object - 要转换的对象。
 * @returns {string} - JSON 字符串或对象的字符串表示。
 */
export const stringifyJson = (object: any) => {
  try {
    return JSON.stringify(object)
  } catch {
    return String(object)
  }
}
